@@ -303,6 +303,12 @@ static void createHeader(int const argc, char* argv[])
 	       << __VERSION__ << endl
 #else
 	       << "UNKNOWN" << endl
+#endif
+	   << "Compiled mode: "
+#ifdef DEBUG
+			<< "Debug build" << endl
+#else
+			<< "Release build" << endl
 #endif
 	   << "Executed on: " << getCurTime() << endl
 	   << getSysinfo() << endl
