@@ -2,8 +2,8 @@
 	This file is part of Warzone 2100.
 	Copyright (C) 1997-XXXX  Jos Fonseca <j_r_fonseca@yahoo.co.uk>
 	 * Originally based on Matt Pietrek's MSJEXHND.CPP in Microsoft Systems Journal, April 1997.
-	Copyright (C) 2008       Giel van Schijndel
-	Copyright (C) 2008       Warzone Resurrection Project
+	Copyright (C) 2008  Giel van Schijndel
+	Copyright (C) 2008-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -867,7 +867,7 @@ void GenerateExceptionReport(PEXCEPTION_POINTERS pExceptionInfo)
 	dbgDumpHeader(hReportFile);
 
 	// First print information about the type of fault
-	rprintf(_T("%s caused "),  GetModuleFileName(NULL, szModule, MAX_PATH) ? szModule : "Application");
+	rprintf(_T("\r\n%s caused "),  GetModuleFileName(NULL, szModule, MAX_PATH) ? szModule : "Application");
 	switch(pExceptionRecord->ExceptionCode)
 	{
 		case EXCEPTION_ACCESS_VIOLATION:
