@@ -50,6 +50,7 @@ static const struct
 //	{ "en", N_("English"), LANG_ENGLISH, SUBLANG_DEFAULT },
 	{ "en_GB", N_("English (United Kingdom)"), LANG_ENGLISH, SUBLANG_DEFAULT },
 	{ "es", N_("Spanish"), LANG_SPANISH, SUBLANG_DEFAULT },
+	{ "et_EE", N_("Estonian"), LANG_ESTONIAN, SUBLANG_DEFAULT },
 //	{ "eu", N_("Basque"), LANG_BASQUE, SUBLANG_DEFAULT },
 	{ "fi", N_("Finnish"), LANG_FINNISH, SUBLANG_DEFAULT },
 	{ "fr", N_("French"), LANG_FRENCH, SUBLANG_DEFAULT },
@@ -103,6 +104,7 @@ static const struct
 //	{ "en", N_("English"), "en_US.UTF-8", "en_US" },
 	{ "en_GB", N_("English (United Kingdom)"), "en_GB.UTF-8", "en_GB" },
 	{ "es", N_("Spanish"), "es.UTF-8", "es" },
+	{ "et_EE", N_("Estonian"), "et_EE.UTF-8", "et_EE" },
 //	{ "eu", N_("Basque"), "eu.UTF-8", "eu" },
 	{ "fi", N_("Finnish"), "fi.UTF-8", "fi" },
 	{ "fr", N_("French"), "fr_FR.UTF-8", "fr_FR" },
