@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -30,8 +30,7 @@
 #define _pieTypes_h
 
 #include "lib/framework/frame.h"
-
-#include "pievector.h"
+#include "lib/framework/vector.h"
 
 //*************************************************************************
 //
