@@ -137,7 +137,7 @@ BOOL screenInitialise(
 				break;
 			case 16:
 				debug( LOG_ERROR, "Warning: Using colour depth of %i instead of %i.", bpp, screenDepth );
-				debug( LOG_ERROR, "         You will experience graphics glitches!" );
+				debug( LOG_ERROR, "You will experience graphics glitches or crashes!" );
 				SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
 				SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 6 );
 				SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
@@ -156,13 +156,16 @@ BOOL screenInitialise(
 	}
 
 	screen = SDL_SetVideoMode(width, height, bpp, video_flags);
-	if ( !screen ) {
+	if ( !screen )
+	{
 		debug( LOG_ERROR, "Error: SDL_SetVideoMode failed (%s).", SDL_GetError() );
 		return false;
 	}
 	if ( SDL_GL_GetAttribute(SDL_GL_DOUBLEBUFFER, &value) == -1)
 	{
 		debug( LOG_ERROR, "OpenGL initialization did not give double buffering!" );
+		debug( LOG_ERROR, "Double buffering is required for this game!");
+		exit(1);
 	}
 	// Note that no initialisation of GLee is required, since this is handled automatically.
 
@@ -176,6 +179,10 @@ BOOL screenInitialise(
 		addDumpInfo(buf);
 		ssprintf(buf, "OpenGL Version : %s", glGetString(GL_VERSION));
 		addDumpInfo(buf);
+		ssprintf(buf, "OpenGL GLSL Version : %s", glGetString(GL_SHADING_LANGUAGE_VERSION));
+		addDumpInfo(buf);
+		ssprintf(buf, "Video Mode %d x %d (%d bpp) (%s)", width, height, bpp, fullScreen ? "fullscreen" : "window");
+		addDumpInfo(buf);
 		/* Dump information about OpenGL implementation to the console */
 		debug(LOG_3D, "OpenGL Vendor : %s", glGetString(GL_VENDOR));
 		debug(LOG_3D, "OpenGL Renderer : %s", glGetString(GL_RENDERER));
@@ -191,10 +198,14 @@ BOOL screenInitialise(
 		debug(LOG_3D, "  * OpenGL 3.0 %s supported!", GLEE_VERSION_3_0 ? "is" : "is NOT");
 		debug(LOG_3D, "  * Texture compression %s supported.", GLEE_ARB_texture_compression ? "is" : "is NOT");
 		debug(LOG_3D, "  * Two side stencil %s supported.", GLEE_EXT_stencil_two_side ? "is" : "is NOT");
+		debug(LOG_3D, "  * ATI separate stencil is%s supported.", GLEE_ATI_separate_stencil ? "" : " NOT");
 		debug(LOG_3D, "  * Stencil wrap %s supported.", GLEE_EXT_stencil_wrap ? "is" : "is NOT");
 		debug(LOG_3D, "  * Anisotropic filtering %s supported.", GLEE_EXT_texture_filter_anisotropic ? "is" : "is NOT");
 		debug(LOG_3D, "  * Rectangular texture %s supported.", GLEE_ARB_texture_rectangle ? "is" : "is NOT");
-		debug(LOG_3D, "  * FrameBuffer Object (FBO) %s supported.", GLEE_EXT_framebuffer_object  ? "is" : "is NOT");
+		debug(LOG_3D, "  * FrameBuffer Object (FBO) %s supported.", GLEE_EXT_framebuffer_object ? "is" : "is NOT");
+		debug(LOG_3D, "  * Shader Objects %s supported.", GL_ARB_shader_objects ? "is" : "is NOT");
+		debug(LOG_3D, "  * Vertex Buffer Object (VBO) %s supported.", GL_ARB_vertex_buffer_object ? "is" : "is NOT");
+		debug(LOG_3D, "  * OpenGL GLSL Version : %s", glGetString(GL_SHADING_LANGUAGE_VERSION));
 	}
 
 	glViewport(0, 0, width, height);
