@@ -164,9 +164,9 @@ static bool playerPasswordFlag[MAX_PLAYERS] = {false};		// we kick on false
  **			   ie ("trunk", "2.1.3", ...)
  ************************************************************************************
 **/
-char VersionString[VersionStringSize] = "2.2"; 
+char VersionString[VersionStringSize] = "2.2.4"; 
 static int NETCODE_VERSION_MAJOR = 2;	// unused for now
-static int NETCODE_VERSION_MINOR = 2;	// unused for now
+static int NETCODE_VERSION_MINOR = 24;	// unused for now
 static int NUMBER_OF_MODS = 0;			// unused for now
 static int NETCODE_HASH = 0;			// unused for now
 
@@ -1730,7 +1730,7 @@ BOOL NETsetupTCPIP(const char *machine)
 // File Transfer programs.
 /** Send file. It returns % of file sent when 100 it's complete. Call until it returns 100. 
  * @TODO Needs to be rewritten. See issue #215. */
-#define MAX_FILE_TRANSFER_PACKET 256
+#define MAX_FILE_TRANSFER_PACKET 1024
 UBYTE NETsendFile(BOOL newFile, char *fileName, UDWORD player)
 {
 	static int32_t  	fileSize,currPos;
@@ -1827,6 +1827,12 @@ UBYTE NETrecvFile(void)
 		pFileHandle = PHYSFS_openWrite(fileName);	// create a new file.
 	}
 
+	if (!pFileHandle) // file can't be opened
+	{
+		debug(LOG_ERROR, "Fatal error while creating file: %s", PHYSFS_getLastError());
+		abort();
+	}
+
 	NETbin(outBuff, bytesRead);
 	NETend();
 
