@@ -66,18 +66,127 @@
 
 #include "multiplay.h"
 #include "lib/netplay/netplay.h"
-
+#include <SDL.h>
 /**********************************************************
  *
  * Local Variables
  *
  *********************************************************/
+void calcDataHash(uint8_t *pBuffer, uint32_t size, uint32_t index);
+UDWORD	hashBuffer(uint8_t *pData, uint32_t size);
 
 // whether a save game is currently being loaded
 static BOOL saveFlag = false;
 
 extern int scr_lineno;
 
+uint32_t	DataHash[DATA_MAXDATA]= {0};
+
+/**
+*	hashBuffer()
+*	\param pData pointer to our buffer
+*	\param size the size of the buffer
+*	\return hash calculated from the buffer.
+*
+*	Note, this is obviously not a very complex hash routine.  This most likely has many collisions possible.
+*	The conversion from CRLF | CR to LF is more complex. :P
+*	This is almost the same routine that Pumpkin had, minus the ugly bug :)
+*/
+UDWORD	hashBuffer(uint8_t *pData, uint32_t size)
+{
+	uint32_t hashval = 0,*val;
+	uint32_t pt = 0, newsize, i;
+	int fillbytes = 0, CRtoStrip = 0;
+	uint8_t *NewData = NULL;
+
+	// find out how many CRs are in the buffer
+	for (i=0; i < size; i++)
+	{
+		if (pData[i] == '\r')
+		{
+			CRtoStrip++;
+		}
+	}
+
+	fillbytes = (size - CRtoStrip) % 4;
+	if (fillbytes == 0)
+	{
+		newsize = size - CRtoStrip;	//don't need to do anything
+	}
+	else
+	{
+		newsize = (size - CRtoStrip) + (4- fillbytes);
+		fillbytes = newsize % 4;
+		debug(LOG_NET, "The size of the buffer (%u bytes) is not on a 4 byte boundry, compensating to a new buffer size of %u bytes.", size, newsize);
+	}
+
+	NewData = malloc(newsize * sizeof(uint8_t));
+	if (!NewData)
+	{
+		//fatal error...
+		debug(LOG_ERROR, "Out of memory!");
+		exit(-1);
+	}
+	memset(NewData, 0xff, newsize);		// fill the new buffer with bit pattern 0xff
+
+	// convert CRLF (windows) | CR (mac OS9?) to LF
+	for(i = 0; i < size- CRtoStrip; i++, pData++)
+	{
+		if (*(pData) == '\r' && *(pData+1) == '\n')
+		{
+			NewData[i] = *(++pData);						// for windows change CRLF to LF
+		}
+		else if (*(pData) == '\r' && *(pData+1) != '\n')
+		{
+			NewData[i] = '\n';								// for mac change CR to LF
+		}
+		else
+		{
+			NewData[i] = *(pData);							// straight copy
+		}
+	}
+
+	debug(LOG_NEVER, "NewData is {%.10s}\n size is %u bytes \n", NewData, newsize );	// this is a bit spammy...
+
+	while (pt < newsize )
+	{
+		val = (uint32_t *)(NewData+pt);
+		hashval = SDL_SwapBE32(hashval ^ SDL_SwapBE32((*val)));		// I hope this is correct...can't test since no PPC machine
+		// spams a ton--but useful for debugging.
+		//	debug(LOG_NET, "hash %08x pt %08x val is %08x", hashval, pt, *val);
+		pt += 4;
+	}
+
+	if (fillbytes)
+	{
+		free(NewData);
+	}
+
+	return hashval;
+}
+
+void calcDataHash(uint8_t *pBuffer, uint32_t size, uint32_t index)
+{
+	if (!bMultiPlayer)
+	{
+		return;
+	}
+
+	// create the hash for that data block.
+	DataHash[index] = SDL_SwapBE32(DataHash[index] ^ hashBuffer(pBuffer, size));	// check endian issues?
+	debug(LOG_NET, "DataHash[%2u] = %08x\n", index, DataHash[index]); 
+	return;
+}
+
+void resetDataHash(void)
+{
+	UDWORD i;
+	for (i = 0; i < DATA_MAXDATA; i++)
+	{
+		DataHash[i] = 0;
+	}
+}
+
 /**********************************************************/
 
 
@@ -93,6 +202,8 @@ void dataClearSaveFlag(void)
 /* Load the body stats */
 static BOOL bufferSBODYLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SBODY);
+
 	if (!loadBodyStats(pBuffer, size)
 	 || !allocComponentList(COMP_BODY, numBodyStats))
 	{
@@ -114,6 +225,8 @@ static void dataReleaseStats(WZ_DECL_UNUSED void *pData)
 /* Load the weapon stats */
 static BOOL bufferSWEAPONLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SWEAPON);
+
 	if (!loadWeaponStats(pBuffer, size)
 	 || !allocComponentList(COMP_WEAPON, numWeaponStats))
 	{
@@ -128,6 +241,8 @@ static BOOL bufferSWEAPONLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the constructor stats */
 static BOOL bufferSCONSTRLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SCONSTR);
+
 	if (!loadConstructStats(pBuffer, size)
 	 || !allocComponentList(COMP_CONSTRUCT, numConstructStats))
 	{
@@ -142,6 +257,8 @@ static BOOL bufferSCONSTRLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the ECM stats */
 static BOOL bufferSECMLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SECM);
+
 	if (!loadECMStats(pBuffer, size)
 	 || !allocComponentList(COMP_ECM, numECMStats))
 	{
@@ -156,6 +273,8 @@ static BOOL bufferSECMLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the Propulsion stats */
 static BOOL bufferSPROPLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SPROP);
+
 	if (!loadPropulsionStats(pBuffer, size)
 	 || !allocComponentList(COMP_PROPULSION, numPropulsionStats))
 	{
@@ -170,6 +289,8 @@ static BOOL bufferSPROPLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the Sensor stats */
 static BOOL bufferSSENSORLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SSENSOR);
+
 	if (!loadSensorStats(pBuffer, size)
 	 || !allocComponentList(COMP_SENSOR, numSensorStats))
 	{
@@ -184,6 +305,8 @@ static BOOL bufferSSENSORLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the Repair stats */
 static BOOL bufferSREPAIRLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SREPAIR);
+
 	if (!loadRepairStats(pBuffer, size)
 	 || !allocComponentList(COMP_REPAIRUNIT, numRepairStats))
 	{
@@ -198,6 +321,8 @@ static BOOL bufferSREPAIRLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the Brain stats */
 static BOOL bufferSBRAINLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SBRAIN);
+
 	if (!loadBrainStats(pBuffer, size)
 	 || !allocComponentList(COMP_BRAIN, numBrainStats))
 	{
@@ -212,6 +337,8 @@ static BOOL bufferSBRAINLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the PropulsionType stats */
 static BOOL bufferSPROPTYPESLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SPROPTY);
+
 	if (!loadPropulsionTypes(pBuffer, size))
 	{
 		return false;
@@ -252,6 +379,8 @@ static BOOL bufferSSPECABILLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the STERRTABLE stats */
 static BOOL bufferSTERRTABLELoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_STERRT);
+
 	if (!loadTerrainTable(pBuffer, size))
 	{
 		return false;
@@ -291,6 +420,8 @@ static BOOL bufferSWEAPSNDLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the Weapon Effect modifier stats */
 static BOOL bufferSWEAPMODLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SWEAPMOD);
+
 	if (!loadWeaponModifiers(pBuffer, size))
 	{
 		return false;
@@ -305,6 +436,8 @@ static BOOL bufferSWEAPMODLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the Template stats */
 static BOOL bufferSTEMPLLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_STEMP);
+
 	if (!loadDroidTemplates(pBuffer, size))
 	{
 		return false;
@@ -325,6 +458,8 @@ static void dataSTEMPLRelease(WZ_DECL_UNUSED void *pData)
 /* Load the Template weapons stats */
 static BOOL bufferSTEMPWEAPLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_STEMPWEAP);
+
 	if (!loadDroidWeapons(pBuffer, size))
 	{
 		return false;
@@ -338,6 +473,8 @@ static BOOL bufferSTEMPWEAPLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the Structure stats */
 static BOOL bufferSSTRUCTLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SSTRUCT);
+
 	if (!loadStructureStats(pBuffer, size)
 	 || !allocStructLists())
 	{
@@ -359,6 +496,8 @@ static void dataSSTRUCTRelease(WZ_DECL_UNUSED void *pData)
 /* Load the Structure Weapons stats */
 static BOOL bufferSSTRWEAPLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SSTRWEAP);
+
 	if (!loadStructureWeapons(pBuffer, size))
 	{
 		return false;
@@ -372,6 +511,8 @@ static BOOL bufferSSTRWEAPLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the Structure Functions stats */
 static BOOL bufferSSTRFUNCLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_STRFUNC);
+
 	if (!loadStructureFunctions(pBuffer, size))
 	{
 		return false;
@@ -385,6 +526,8 @@ static BOOL bufferSSTRFUNCLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the Structure strength modifier stats */
 static BOOL bufferSSTRMODLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SSTRMOD);
+
 	if (!loadStructureStrengthModifiers(pBuffer, size))
 	{
 		return false;
@@ -398,6 +541,8 @@ static BOOL bufferSSTRMODLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the Feature stats */
 static BOOL bufferSFEATLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SFEAT);
+
 	if (!loadFeatureStats(pBuffer, size))
 	{
 		return false;
@@ -417,6 +562,8 @@ static void dataSFEATRelease(WZ_DECL_UNUSED void *pData)
 /* Load the Functions stats */
 static BOOL bufferSFUNCLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_SFUNC);
+
 	if (!loadFunctionStats(pBuffer, size))
 	{
 		return false;
@@ -446,6 +593,8 @@ static void dataRESCHRelease(WZ_DECL_UNUSED void *pData)
 /* Load the Research stats */
 static BOOL bufferRESCHLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_RESCH);
+
 	//check to see if already loaded
 	if (numResearch > 0)
 	{
@@ -470,6 +619,8 @@ static BOOL bufferRESCHLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the research pre-requisites */
 static BOOL bufferRPREREQLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_RPREREQ);
+
 	if (!loadResearchPR(pBuffer, size))
 	{
 		return false;
@@ -483,6 +634,8 @@ static BOOL bufferRPREREQLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the research components made redundant */
 static BOOL bufferRCOMPREDLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_RCOMPRED);
+
 	if (!loadResearchArtefacts(pBuffer, size, RED_LIST))
 	{
 		return false;
@@ -496,6 +649,8 @@ static BOOL bufferRCOMPREDLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the research component results */
 static BOOL bufferRCOMPRESLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_RCOMPRES);
+
 	if (!loadResearchArtefacts(pBuffer, size, RES_LIST))
 	{
 		return false;
@@ -509,6 +664,8 @@ static BOOL bufferRCOMPRESLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the research structures required */
 static BOOL bufferRSTRREQLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_RSTRREQ);
+
 	if (!loadResearchStructures(pBuffer, size, REQ_LIST))
 	{
 		return false;
@@ -522,6 +679,8 @@ static BOOL bufferRSTRREQLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the research structures made redundant */
 static BOOL bufferRSTRREDLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_RSTRRED);
+
 	if (!loadResearchStructures(pBuffer, size, RED_LIST))
 	{
 		return false;
@@ -535,6 +694,8 @@ static BOOL bufferRSTRREDLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the research structure results */
 static BOOL bufferRSTRRESLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_RSTRRES);
+
 	if (!loadResearchStructures(pBuffer, size, RES_LIST))
 	{
 		return false;
@@ -548,6 +709,8 @@ static BOOL bufferRSTRRESLoad(const char *pBuffer, UDWORD size, void **ppData)
 /* Load the research functions */
 static BOOL bufferRFUNCLoad(const char *pBuffer, UDWORD size, void **ppData)
 {
+	calcDataHash((uint8_t *)pBuffer, size, DATA_RFUNC);
+
 	if (!loadResearchFunctions(pBuffer, size))
 	{
 		return false;
@@ -841,6 +1004,8 @@ static BOOL dataScriptLoad(const char* fileName, void **ppData)
 	static const bool printHack = false;
 	SCRIPT_CODE** psProg = (SCRIPT_CODE**)ppData;
 	PHYSFS_file* fileHandle;
+	uint8_t *pBuffer;
+	PHYSFS_sint64 fileSize = 0;
 
 	debug(LOG_WZ, "COMPILING SCRIPT ...%s", GetLastResourceFilename());
 	scr_lineno = 1;
@@ -851,6 +1016,22 @@ static BOOL dataScriptLoad(const char* fileName, void **ppData)
 	{
 		return false;
 	}
+	// due to the changes in r2531 we must do this routine a bit different.
+	fileSize = PHYSFS_fileLength(fileHandle);
+
+	pBuffer = malloc(fileSize * sizeof(char));
+	if (pBuffer == NULL)
+	{
+		debug(LOG_ERROR, "Fatal memory allocation, couldn't allocate %lld buffer", fileSize);
+		abort();
+	}
+
+	PHYSFS_read(fileHandle, pBuffer, 1, fileSize);
+
+	calcDataHash(pBuffer, fileSize, DATA_SCRIPT);
+
+	free(pBuffer);
+	PHYSFS_seek(fileHandle, 0);		//reset position
 
 	*psProg = scriptCompile(fileHandle, SCRIPTTYPE);
 
@@ -883,6 +1064,8 @@ static BOOL dataScriptLoadVals(const char* fileName, void **ppData)
 {
 	BOOL success;
 	PHYSFS_file* fileHandle;
+	uint8_t *pBuffer;
+	PHYSFS_sint64 fileSize = 0;
 
 	*ppData = NULL;
 
@@ -900,6 +1083,23 @@ static BOOL dataScriptLoadVals(const char* fileName, void **ppData)
 	{
 		return false;
 	}
+	// due to the changes in r2532 we must do this routine a bit different.
+	fileSize = PHYSFS_fileLength(fileHandle);
+
+	pBuffer = malloc(fileSize * sizeof(char));
+	if (pBuffer == NULL)
+	{
+		debug(LOG_ERROR, "Fatal memory allocation, couldn't allocate %lld buffer", fileSize);
+		abort();
+	}
+
+	PHYSFS_read(fileHandle, pBuffer, 1, fileSize);
+
+	calcDataHash(pBuffer, fileSize, DATA_SCRIPTVAL);
+
+	free(pBuffer);
+
+	PHYSFS_seek(fileHandle, 0);		//reset position
 
 	success = scrvLoad(fileHandle);
 
@@ -985,6 +1185,8 @@ static const RES_TYPE_MIN_FILE FileResourceTypes[] =
 /* Pass all the data loading functions to the framework library */
 BOOL dataInitLoadFuncs(void)
 {
+	// init the data integrity hash;
+	resetDataHash();
 	// Using iterator style: begin iterator (ResourceTypes),
 	// end iterator (EndType), and current iterator (CurrentType)
 
