@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "lib/framework/frame.h"
 #include "lib/framework/strres.h"
 #include "lib/framework/frameresource.h"
+#include "lib/framework/lexer_input.h"
 #include "message.h"
 #include "stats.h"
 #include "text.h"
@@ -38,12 +39,11 @@
 #include "lib/ivis_common/piedef.h"
 #include "objmem.h"
 #include "map.h"
+#include "message_parser.tab.h"
+#include "messagely.h"
 
 #include "multiplay.h"
 
-//max number of text strings or sequences for viewdata
-#define MAX_DATA		4
-
 //array of pointers for the view data
 static VIEWDATA_LIST		*apsViewData;
 
@@ -67,7 +67,6 @@ static void addProximityDisplay(MESSAGE *psMessage, BOOL proxPos, UDWORD player)
 static void removeProxDisp(MESSAGE *psMessage, UDWORD player);
 static void checkMessages(MSG_VIEWDATA *psViewData);
 
-
 /* Creating a new message
  * new is a pointer to a pointer to the new message
  * type is the type of the message
@@ -76,8 +75,8 @@ static inline MESSAGE* createMessage(MESSAGE_TYPE msgType, UDWORD player)
 {
 	MESSAGE *newMsg;
 
-	ASSERT(player < MAX_PLAYERS, "createMessage: Bad player");
-	ASSERT(msgType < MSG_TYPES, "createMessage: Bad message");
+	ASSERT_OR_RETURN(NULL, player < MAX_PLAYERS, "Bad player");
+	ASSERT_OR_RETURN(NULL, msgType < MSG_TYPES, "Bad message");
 	if (player >= MAX_PLAYERS || msgType >= MSG_TYPES)
 	{
 		return NULL;
@@ -110,8 +109,8 @@ static inline void addMessageToList(MESSAGE *list[MAX_PLAYERS], MESSAGE *msg, UD
 {
 	MESSAGE *psCurr = NULL, *psPrev = NULL;
 
-	ASSERT(msg != NULL, "addMessageToList: Invalid message pointer");
-	ASSERT(player < MAX_PLAYERS, "addMessageToList: Bad player");
+	ASSERT_OR_RETURN( , msg != NULL, "Invalid message pointer");
+	ASSERT_OR_RETURN( , player < MAX_PLAYERS, "Bad player");
 
 	// If there is no message list, create one
 	if (list[player] == NULL)
@@ -123,14 +122,14 @@ static inline void addMessageToList(MESSAGE *list[MAX_PLAYERS], MESSAGE *msg, UD
 	}
 
 	switch (msg->type)
-       	{
+		{
 		case MSG_CAMPAIGN:
 			/*add it before the first mission/research/prox message */
 			for(psCurr = list[player]; psCurr != NULL; psCurr = psCurr->psNext)
 			{
 				if (psCurr->type == MSG_MISSION ||
-				    psCurr->type == MSG_RESEARCH ||
-				    psCurr->type == MSG_PROXIMITY)
+					psCurr->type == MSG_RESEARCH ||
+					psCurr->type == MSG_PROXIMITY)
 					break;
 
 				psPrev = psCurr;
@@ -188,7 +187,7 @@ static inline void addMessageToList(MESSAGE *list[MAX_PLAYERS], MESSAGE *msg, UD
 
 			break;
 		default:
-			debug(LOG_ERROR, "addMessageToList: unknown message type");
+			debug(LOG_ERROR, "unknown message type");
 			break;
 	}
 }
@@ -203,8 +202,8 @@ static inline void removeMessageFromList(MESSAGE *list[], MESSAGE *del, UDWORD p
 {
 	MESSAGE *psPrev = NULL, *psCurr;
 
-	ASSERT(del != NULL, "removeMessageFromList: Invalid message pointer");
-	ASSERT(player < MAX_PLAYERS, "removeMessageFromList: Bad player");
+	ASSERT_OR_RETURN( , del != NULL, "Invalid message pointer");
+	ASSERT_OR_RETURN( , player < MAX_PLAYERS, "Bad player");
 
 	// If the message to remove is the first one in the list then mark the next one as the first
 	if (list[player] == del)
@@ -220,7 +219,7 @@ static inline void removeMessageFromList(MESSAGE *list[], MESSAGE *del, UDWORD p
 		psPrev = psCurr;
 	}
 
-	ASSERT(psCurr != NULL, "removeMessage: message not found in list");
+	ASSERT(psCurr != NULL, "message not found in list");
 
 	if (psCurr != NULL)
 	{
@@ -242,7 +241,7 @@ static inline void releaseAllMessages(MESSAGE *list[])
 		// Iterate through all messages in list
 		for(psCurr = list[i]; psCurr != NULL; psCurr = psNext)
 		{
-	 		psNext = psCurr->psNext;
+			psNext = psCurr->psNext;
 			free(psCurr);
 		}
 		list[i] = NULL;
@@ -277,7 +276,7 @@ MESSAGE * addBeaconMessage(MESSAGE_TYPE msgType, BOOL proxPos, UDWORD player)
 {
 	MESSAGE* psBeaconMsgToAdd = addMessage(msgType, proxPos, player);
 
-	ASSERT(psBeaconMsgToAdd, "addBeaconMessage: createMessage failed");
+	ASSERT_OR_RETURN(NULL, psBeaconMsgToAdd, "createMessage failed");
 
 	// remember we are storing beacon data in this message
 	psBeaconMsgToAdd->dataType = MSG_DATA_BEACON;
@@ -316,14 +315,14 @@ static void addProximityDisplay(MESSAGE *psMessage, BOOL proxPos, UDWORD player)
 {
 	PROXIMITY_DISPLAY *psToAdd;
 
-	ASSERT(player < MAX_PLAYERS, "addProximityDisplay: Bad player");
+	ASSERT_OR_RETURN( , player < MAX_PLAYERS, "Bad player");
 	debug(LOG_MSG, "Added prox display for player %u (proxPos=%d)", player, (int)proxPos);
 
 	//create the proximity display
 	psToAdd = (PROXIMITY_DISPLAY*)malloc(sizeof(PROXIMITY_DISPLAY));
 	if (psToAdd == NULL)
 	{
-		ASSERT(false, "addProximityDisplay: out of memory");
+		ASSERT(false, "out of memory");
 		return;
 	}
 
@@ -339,8 +338,6 @@ static void addProximityDisplay(MESSAGE *psMessage, BOOL proxPos, UDWORD player)
 	psToAdd->screenX = 0;
 	psToAdd->screenY = 0;
 	psToAdd->screenR = 0;
-	psToAdd->radarX = 0;
-	psToAdd->radarY = 0;
 	psToAdd->player = player;
 	psToAdd->timeLastDrawn = 0;
 	psToAdd->frameNumber = 0;
@@ -356,12 +353,12 @@ static void addProximityDisplay(MESSAGE *psMessage, BOOL proxPos, UDWORD player)
 	currentNumProxDisplays++;
 }
 
- /*remove a message */
+/*remove a message */
 void removeMessage(MESSAGE *psDel, UDWORD player)
 {
-	ASSERT(player < MAX_PLAYERS, "removeMessage: Bad player");
-	ASSERT(psDel != NULL, "removeMessage: Bad message");
-	debug(LOG_MSG, "removeMessage: removing message for player %d", player);
+	ASSERT_OR_RETURN( , player < MAX_PLAYERS, "Bad player");
+	ASSERT_OR_RETURN( , psDel != NULL, "Bad message");
+	debug(LOG_MSG, "removing message for player %d", player);
 
 	if (psDel->type == MSG_PROXIMITY)
 	{
@@ -375,8 +372,8 @@ void removeProxDisp(MESSAGE *psMessage, UDWORD player)
 {
 	PROXIMITY_DISPLAY		*psCurr, *psPrev;
 
-	ASSERT(player < MAX_PLAYERS, "removeProxDisp: Bad player");
-	ASSERT(psMessage != NULL, "removeProxDisp: Bad message");
+	ASSERT_OR_RETURN( , player < MAX_PLAYERS, "Bad player");
+	ASSERT_OR_RETURN( , psMessage != NULL, "Bad message");
 
 	//find the proximity display for this message
 	if (apsProxDisp[player]->psMessage == psMessage)
@@ -423,7 +420,7 @@ void releaseAllProxDisp(void)
 	{
 		for(psCurr = apsProxDisp[player]; psCurr != NULL; psCurr = psNext)
 		{
-	 		psNext = psCurr->psNext;
+			psNext = psCurr->psNext;
 			//remove message associated with this display
 			removeMessage(psCurr->psMessage, player);
 		}
@@ -447,13 +444,13 @@ BOOL initMessage(void)
 	return true;
 }
 
-static BOOL addToViewDataList(VIEWDATA *psViewData, UBYTE numData)
+bool addToViewDataList(VIEWDATA* psViewData, unsigned int numData)
 {
 	VIEWDATA_LIST		*psAdd = (VIEWDATA_LIST*)malloc(sizeof(VIEWDATA_LIST));
 
 	if (psAdd == NULL)
 	{
-		ASSERT(false, "addToViewDataList: out of memory");
+		ASSERT(false, "out of memory");
 		return false;
 	}
 
@@ -469,7 +466,7 @@ static BOOL addToViewDataList(VIEWDATA *psViewData, UBYTE numData)
 /*load the view data for the messages from the file */
 VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 {
-	UDWORD				i, id, dataInc, seqInc, numFrames, numData, count, count2;
+	UDWORD				i, dataInc, seqInc, dummy, numData, count, count2;
 	VIEWDATA			*psViewData, *pData;
 	VIEW_RESEARCH		*psViewRes;
 	VIEW_REPLAY			*psViewReplay;
@@ -479,12 +476,12 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 	char				audioName[MAX_STR_LENGTH];
 	SDWORD				LocX,LocY,LocZ, audioID;
 	PROX_TYPE	proxType;
-        int cnt;
+	int cnt;
 
 	numData = numCR(pViewMsgData, bufferSize);
 	if (numData > UBYTE_MAX)
 	{
-		ASSERT(false, "loadViewData: Didn't expect 256 viewData messages!");
+		ASSERT(false, "Didn't expect 256 viewData messages!");
 		return NULL;
 	}
 
@@ -513,7 +510,7 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 
 		//read the data into the storage - the data is delimeted using comma's
 		sscanf(pViewMsgData,"%[^','],%d%n",name, &numText,&cnt);
-                pViewMsgData += cnt;
+		pViewMsgData += cnt;
 
 		//check not loading up too many text strings
 		if (numText > MAX_DATA)
@@ -524,13 +521,12 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 		psViewData->numText=(UBYTE)numText;
 
 		//allocate storage for the name
- 		psViewData->pName = malloc(strlen(name) + 1);
+		psViewData->pName = strdup(name);
 		if (psViewData->pName == NULL)
 		{
 			ASSERT(false, "Out of memory");
 			return NULL;
 		}
-		strcpy(psViewData->pName, name);
 		debug(LOG_MSG, "Loaded %s", psViewData->pName);
 
 		//allocate space for text strings
@@ -544,21 +540,20 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 		{
 			name[0] = '\0';
 			sscanf(pViewMsgData,",%[^',']%n",name,&cnt);
-                        pViewMsgData += cnt;
+			pViewMsgData += cnt;
 
-			//get the ID for the string
-			if (!strresGetIDNum(psStringRes, name, &id))
+			// Get the string from the ID string
+			psViewData->ppTextMsg[dataInc] = strresGetString(psStringRes, name);
+			if (!psViewData->ppTextMsg[dataInc])
 			{
-				ASSERT(false, "Cannot find the view data string id %s ", name);
+				ASSERT(!"Cannot find string resource", "Cannot find the view data string with id \"%s\"", name);
 				return NULL;
 			}
-			//get the string from the id
-			psViewData->ppTextMsg[dataInc] = strresGetString(psStringRes, id);
 		}
 
 		sscanf(pViewMsgData, ",%d%n", &readint, &cnt);
 		psViewData->type = readint;
-                pViewMsgData += cnt;
+		pViewMsgData += cnt;
 
 		//allocate data according to type
 		switch (psViewData->type)
@@ -575,13 +570,13 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 			string[0] = '\0';
 			audioName[0] = '\0';
 			sscanf(pViewMsgData,",%[^','],%[^','],%[^','],%[^','],%d%n",
-				imdName, imdName2, string, audioName, &numFrames,&cnt);
-                        pViewMsgData += cnt;
+				imdName, imdName2, string, audioName, &dummy, &cnt);
+			pViewMsgData += cnt;
 			psViewRes = (VIEW_RESEARCH *)psViewData->pData;
 			psViewRes->pIMD = (iIMDShape *) resGetData("IMD", imdName);
 			if (psViewRes->pIMD == NULL)
 			{
-				ASSERT(LOG_ERROR, "Cannot find the PIE for message %s", name);
+				ASSERT(false, "Cannot find the PIE for message %s", name);
 				return NULL;
 			}
 			if (strcmp(imdName2, "0"))
@@ -597,25 +592,22 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 			{
 				psViewRes->pIMD2 = NULL;
 			}
-			strcpy(psViewRes->sequenceName, string);
+			sstrcpy(psViewRes->sequenceName, string);
 			//get the audio text string
 			if (strcmp(audioName, "0"))
 			{
 				//allocate space
-				psViewRes->pAudio = (char *) malloc(strlen(audioName) + 1);
+				psViewRes->pAudio = strdup(audioName);
 				if (psViewRes->pAudio == NULL)
 				{
 					ASSERT(false, "loadViewData - Out of memory");
 					return NULL;
 				}
-				strcpy(psViewRes->pAudio, audioName);
 			}
 			else
 			{
 				psViewRes->pAudio = NULL;
 			}
-			//this is for the PSX only
-			psViewRes->numFrames = (UWORD)numFrames;
 			break;
 		case VIEW_RPL:
 		case VIEW_RPLX:
@@ -632,11 +624,11 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 			//read in number of sequences for this message
 			//sscanf(pViewMsgData, "%d", &psViewReplay->numSeq);
 			sscanf(pViewMsgData, ",%d%n", &count,&cnt);
-                        pViewMsgData += cnt;
+			pViewMsgData += cnt;
 
 			if (count > MAX_DATA)
 			{
-				ASSERT(false, "loadViewData: too many sequence for %s", psViewData->pName);
+				ASSERT(false, "too many sequence for %s", psViewData->pName);
 				return NULL;
 			}
 
@@ -654,10 +646,10 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 				if (psViewData->type == VIEW_RPL)
 				{
 					sscanf(pViewMsgData, ",%[^','],%d%n", name, &count,&cnt);
-                                        pViewMsgData += cnt;
+					pViewMsgData += cnt;
 					if (count > MAX_DATA)
 					{
-						ASSERT(false, "loadViewData: too many strings for %s", psViewData->pName);
+						ASSERT(false, "too many strings for %s", psViewData->pName);
 						return NULL;
 					}
 					psViewReplay->pSeqList[dataInc].numText = (UBYTE)count;
@@ -667,17 +659,17 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 				else //extended type
 				{
 					sscanf(pViewMsgData, ",%[^','],%d,%d%n", name, &count,	&count2,&cnt);
-                                        pViewMsgData += cnt;
+					pViewMsgData += cnt;
 					if (count > MAX_DATA)
 					{
-						ASSERT(false, "loadViewData: invalid video playback flag %s", psViewData->pName);
+						ASSERT(false, "invalid video playback flag %s", psViewData->pName);
 						return NULL;
 					}
 					psViewReplay->pSeqList[dataInc].flag = (UBYTE)count;
 					//check not loading up too many text strings
 					if (count2 > MAX_DATA)
 					{
-						ASSERT(false, "loadViewData: too many text strings for seq for %s", psViewData->pName);
+						ASSERT(false, "too many text strings for seq for %s", psViewData->pName);
 						return NULL;
 					}
 					psViewReplay->pSeqList[dataInc].numText = (UBYTE)count2;
@@ -688,7 +680,7 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 				//allocate space for text strings
 				if (psViewReplay->pSeqList[dataInc].numText)
 				{
-					psViewReplay->pSeqList[dataInc].ppTextMsg = (char **) malloc(
+					psViewReplay->pSeqList[dataInc].ppTextMsg = (const char **) malloc(
 						psViewReplay->pSeqList[dataInc].numText * sizeof(char *));
 				}
 				//read in the data for the text strings
@@ -697,36 +689,29 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 				{
 					name[0] = '\0';
 					sscanf(pViewMsgData,",%[^',']%n", name,&cnt);
-                                        pViewMsgData += cnt;
-					//get the ID for the string
-					if (!strresGetIDNum(psStringRes, name, &id))
+					pViewMsgData += cnt;
+
+					// Get the string from the ID string
+					psViewReplay->pSeqList[dataInc].ppTextMsg[seqInc] = strresGetString(psStringRes, name);
+					if (!psViewReplay->pSeqList[dataInc].ppTextMsg[seqInc])
 					{
-						ASSERT(false, "Cannot find the view data string id %s ", name);
+						ASSERT(!"Cannot find string resource", "Cannot find the view data string with id \"%s\"", name);
 						return NULL;
 					}
-
-					//get the string from the id
-					psViewReplay->pSeqList[dataInc].ppTextMsg[seqInc] = strresGetString(psStringRes, id);
 				}
 				//get the audio text string
-				sscanf(pViewMsgData,",%[^','],%d%n", audioName, &count,&cnt);
-                                pViewMsgData += cnt;
-
-				ASSERT( count < UWORD_MAX, "loadViewData: numFrames too high for %s", name );
-
-				psViewReplay->pSeqList[dataInc].numFrames = (UWORD)count;
+				sscanf(pViewMsgData,",%[^','],%d%n", audioName, &dummy, &cnt);
+				pViewMsgData += cnt;
 
 				if (strcmp(audioName, "0"))
 				{
 					//allocate space
-					psViewReplay->pSeqList[dataInc].pAudio = (char *) malloc(
-						strlen(audioName) + 1);
+					psViewReplay->pSeqList[dataInc].pAudio = strdup(audioName);
 					if (psViewReplay->pSeqList[dataInc].pAudio == NULL)
 					{
 						ASSERT(LOG_ERROR, "loadViewData - Out of memory");
 						return NULL;
 					}
-					strcpy(psViewReplay->pSeqList[dataInc].pAudio, audioName);
 				}
 				else
 				{
@@ -742,12 +727,14 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 			{
 				ASSERT(false, "Unable to allocate memory");
 				return NULL;
-			} else {
+			}
+			else
+			{
 				int tmp;
 
 				audioName[0] = '\0';
 				sscanf( pViewMsgData, ", %d,%d,%d,%[^','],%d%n", &LocX, &LocY, &LocZ,
-				        audioName, &tmp, &cnt);
+						audioName, &tmp, &cnt);
 				proxType = tmp;
 			}
 			pViewMsgData += cnt;
@@ -761,7 +748,7 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 			{
 				if ( (audioID = audio_GetIDFromStr( audioName )) == NO_SOUND )
 				{
-					ASSERT(false, "loadViewData: couldn't get ID %d for weapon sound %s", audioID, audioName);
+					ASSERT(false, "couldn't get ID %d for weapon sound %s", audioID, audioName);
 					return false;
 				}
 
@@ -779,19 +766,19 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 
 			if (LocX < 0)
 			{
-				ASSERT(false, "loadViewData: Negative X coord for prox message - %s",name);
+				ASSERT(false, "Negative X coord for prox message - %s",name);
 				return NULL;
 			}
 			((VIEW_PROXIMITY *)psViewData->pData)->x = (UDWORD)LocX;
 			if (LocY < 0)
 			{
-				ASSERT(false, "loadViewData: Negative Y coord for prox message - %s",name);
+				ASSERT(false, "Negative Y coord for prox message - %s",name);
 				return NULL;
 			}
 			((VIEW_PROXIMITY *)psViewData->pData)->y = (UDWORD)LocY;
 			if (LocZ < 0)
 			{
-				ASSERT(false, "loadViewData: Negative Z coord for prox message - %s",name);
+				ASSERT(false, "Negative Z coord for prox message - %s",name);
 				return NULL;
 			}
 			((VIEW_PROXIMITY *)psViewData->pData)->z = (UDWORD)LocZ;
@@ -816,6 +803,30 @@ VIEWDATA *loadViewData(const char *pViewMsgData, UDWORD bufferSize)
 	return pData;
 }
 
+VIEWDATA* loadResearchViewData(const char* fileName)
+{
+	bool retval;
+	lexerinput_t input;
+	VIEWDATA* psViewData;
+
+	input.type = LEXINPUT_PHYSFS;
+	input.input.physfsfile = PHYSFS_openRead(fileName);
+	debug(LOG_WZ, "Reading...[directory: %s] %s", PHYSFS_getRealDir(fileName), fileName);
+	if (!input.input.physfsfile)
+	{
+		debug(LOG_ERROR, "PHYSFS_openRead(\"%s\") failed with error: %s\n", fileName, PHYSFS_getLastError());
+		return NULL;
+	}
+
+	message_set_extra(&input);
+	retval = (message_parse(&psViewData) == 0);
+
+	message_lex_destroy();
+	PHYSFS_close(input.input.physfsfile);
+
+	return retval ? psViewData : NULL;
+}
+
 /*get the view data identified by the name */
 VIEWDATA * getViewData(const char *pName)
 {
@@ -849,89 +860,76 @@ BOOL messageShutdown(void)
 /* Release the viewdata memory */
 void viewDataShutDown(VIEWDATA *psViewData)
 {
-	VIEWDATA_LIST	*psList, *psPrev;
-	UDWORD			seqInc;
-	VIEW_REPLAY		*psViewReplay;
-	VIEW_RESEARCH	*psViewRes;
-	UBYTE			i;
+	VIEWDATA_LIST	*psList, **psPrev;
 
-	psPrev = apsViewData;
-
-	for (psList = apsViewData; psList != NULL; psList = psList->psNext)
+	for (psList = apsViewData, psPrev = &apsViewData; psList != NULL; psPrev = &psList->psNext, psList = psList->psNext)
 	{
-		if (psList->psViewData == psViewData)
+		unsigned int i;
+
+		// Skip non-matching etnries
+		if (psList->psViewData != psViewData)
 		{
-			for (i = 0; i < psList->numViewData; i++)
-			{
-				psViewData = &psList->psViewData[i];
-
-				//check for any messages using this viewdata
-				checkMessages((MSG_VIEWDATA *)psViewData);
-
-				free(psViewData->pName);
-				psViewData->pName = NULL;
-
-				//free the space allocated for the text messages
-				if (psViewData->numText)
-				{
-					free(psViewData->ppTextMsg);
-					psViewData->ppTextMsg = NULL;
-				}
-
-				//free the space allocated for multiple sequences
-				if (psViewData->type == VIEW_RPL)
-				{
-					psViewReplay = (VIEW_REPLAY *)psViewData->pData;
-					if (psViewReplay->numSeq)
-					{
-						for (seqInc = 0; seqInc < psViewReplay->numSeq; seqInc++)
-						{
-							//free the space allocated for the text messages
-							if (psViewReplay->pSeqList[seqInc].numText)
-							{
-								free(psViewReplay->pSeqList[seqInc].ppTextMsg);
-								psViewReplay->pSeqList[seqInc].ppTextMsg = NULL;
-							}
-							if (psViewReplay->pSeqList[seqInc].pAudio)
-							{
-								free(psViewReplay->pSeqList[seqInc].pAudio);
-								psViewReplay->pSeqList[seqInc].pAudio = NULL;
-							}
-						}
-						free(psViewReplay->pSeqList);
-						psViewReplay->pSeqList = NULL;
-					}
-				}
-				else if (psViewData->type == VIEW_RES)
-				{
-					psViewRes = (VIEW_RESEARCH *)psViewData->pData;
-					if (psViewRes->pAudio)
-					{
-						free(psViewRes->pAudio);
-						psViewRes->pAudio = NULL;
-					}
-				}
-				free(psViewData->pData);
-				psViewData->pData = NULL;
-			}
-			free(psList->psViewData);
-			psList->psViewData = NULL;
-
-			//remove viewData list from the heap
-			if (psList == apsViewData)
-			{
-				apsViewData = psList->psNext;
-				free(psList);
-			}
-			else
-			{
-				psPrev->psNext = psList->psNext;
-				free(psList);
-			}
-			break;
+			continue;
 		}
+
+		for (i = 0; i < psList->numViewData; ++i)
+		{
+			psViewData = &psList->psViewData[i];
+
+			//check for any messages using this viewdata
+			checkMessages((MSG_VIEWDATA *)psViewData);
+
+			free(psViewData->pName);
+
+			//free the space allocated for the text messages
+			if (psViewData->numText)
+			{
+				free(psViewData->ppTextMsg);
+			}
+
+			//free the space allocated for multiple sequences
+			if (psViewData->type == VIEW_RPL)
+			{
+				VIEW_REPLAY* const psViewReplay = (VIEW_REPLAY *)psViewData->pData;
+				if (psViewReplay->numSeq)
+				{
+					unsigned int seqInc;
+					for (seqInc = 0; seqInc < psViewReplay->numSeq; ++seqInc)
+					{
+						//free the space allocated for the text messages
+						if (psViewReplay->pSeqList[seqInc].numText)
+						{
+							free(psViewReplay->pSeqList[seqInc].ppTextMsg);
+						}
+						if (psViewReplay->pSeqList[seqInc].pAudio)
+						{
+							free(psViewReplay->pSeqList[seqInc].pAudio);
+						}
+					}
+					free(psViewReplay->pSeqList);
+				}
+			}
+			else if (psViewData->type == VIEW_RES)
+			{
+				VIEW_RESEARCH* const psViewRes = (VIEW_RESEARCH *)psViewData->pData;
+				if (psViewRes->pAudio)
+				{
+					free(psViewRes->pAudio);
+				}
+			}
+			free(psViewData->pData);
+		}
+		free(psList->psViewData);
+
+		// remove viewData list from the list
+		*psPrev = psList->psNext;
+		free(psList);
+
+		/* Although we're a O(n) algorithm, lets not go for fullblown
+		 * O(n) behaviour if not required.
+		 */
+		break;
 	}
-	psPrev = psList;
 }
 
 /* Looks through the players list of messages to find one with the same viewData
@@ -940,8 +938,8 @@ MESSAGE * findMessage(MSG_VIEWDATA *pViewData, MESSAGE_TYPE type, UDWORD player)
 {
 	MESSAGE					*psCurr;
 
-	ASSERT(player < MAX_PLAYERS, "findMessage: Bad player");
-	ASSERT(type < MSG_TYPES, "removeMessage: Bad message type");
+	ASSERT_OR_RETURN(NULL, player < MAX_PLAYERS, "Bad player");
+	ASSERT_OR_RETURN(NULL , type < MSG_TYPES, "Bad message type");
 
 	for (psCurr = apsMessages[player]; psCurr != NULL; psCurr = psCurr->psNext)
 	{
@@ -982,8 +980,8 @@ void displayProximityMessage(PROXIMITY_DISPLAY *psProxDisp)
 	{
 		FEATURE	*psFeature = (FEATURE *)psProxDisp->psMessage->pViewData;
 
-		ASSERT( ((BASE_OBJECT *)psProxDisp->psMessage->pViewData)->type ==
-			OBJ_FEATURE, "displayProximityMessage: invalid feature" );
+		ASSERT_OR_RETURN( , ((BASE_OBJECT *)psProxDisp->psMessage->pViewData)->type ==
+			OBJ_FEATURE, "invalid feature" );
 
 		if (psFeature->psStats->subType == FEAT_OIL_RESOURCE)
 		{
@@ -1046,29 +1044,29 @@ void checkMessages(MSG_VIEWDATA *psViewData)
 //add proximity messages for all untapped VISIBLE oil resources
 void addOilResourceProximities(void)
 {
-    FEATURE     *psFeat;
-    MESSAGE     *psMessage;
+	FEATURE     *psFeat;
+	MESSAGE     *psMessage;
 
-    //look thru the features to find oil resources
-    for (psFeat = apsFeatureLists[0]; psFeat != NULL; psFeat = psFeat->psNext)
-    {
-        if (psFeat->psStats->subType == FEAT_OIL_RESOURCE)
-        {
-            //check to see if the feature is visible to the selected player
-            if (psFeat->visible[selectedPlayer])
-            {
-                //if there isn't an oil derrick built on it
+	//look thru the features to find oil resources
+	for (psFeat = apsFeatureLists[0]; psFeat != NULL; psFeat = psFeat->psNext)
+	{
+		if (psFeat->psStats->subType == FEAT_OIL_RESOURCE)
+		{
+			//check to see if the feature is visible to the selected player
+			if (psFeat->visible[selectedPlayer])
+			{
+				//if there isn't an oil derrick built on it
 				if (!TileHasStructure(mapTile(map_coord(psFeat->pos.x),
 					map_coord(psFeat->pos.y))))
 				{
-                    //add a proximity message
+					//add a proximity message
 					psMessage = addMessage(MSG_PROXIMITY, true, selectedPlayer);
 					if (psMessage)
 					{
 						psMessage->pViewData = (MSG_VIEWDATA *)psFeat;
 					}
-                }
-            }
-        }
-    }
+				}
+			}
+		}
+	}
 }
