@@ -1,7 +1,7 @@
 /*
 	This file is part of Warzone 2100.
 	Copyright (C) 1999-2004  Eidos Interactive
-	Copyright (C) 2005-2007  Warzone Resurrection Project
+	Copyright (C) 2005-2009  Warzone Resurrection Project
 
 	Warzone 2100 is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ extern char	buildTime[8];
 // dpid == 0 for no new players.
 void sendOptions(uint32_t dest, uint32_t play)
 {
-	int i, j;
+	unsigned int i;
 
 	NETbeginEncode(NET_OPTIONS, NET_ALL_PLAYERS);
 
@@ -112,6 +112,8 @@ void sendOptions(uint32_t dest, uint32_t play)
 	// Same goes for the alliances
 	for (i = 0; i < MAX_PLAYERS; i++)
 	{
+		unsigned int j;
+
 		for (j = 0; j < MAX_PLAYERS; j++)
 		{
 			NETuint8_t(&alliances[i][j]);
@@ -167,7 +169,7 @@ static BOOL checkGameWdg(const char *nm)
 // options for a game. (usually recvd in frontend)
 void recvOptions()
 {
-	int		i, j;
+	unsigned int i;
 	UDWORD	play;
 	UDWORD	newPl;
 
@@ -219,6 +221,8 @@ void recvOptions()
 	// Alliances
 	for (i = 0; i < MAX_PLAYERS; i++)
 	{
+		unsigned int j;
+
 		for (j = 0; j < MAX_PLAYERS; j++)
 		{
 			NETuint8_t(&alliances[i][j]);
@@ -302,7 +306,7 @@ void recvOptions()
 	}
 	else
 	{
-		loadMapPreview();
+		loadMapPreview(false);
 	}
 }
 
@@ -333,7 +337,7 @@ BOOL hostCampaign(char *sGame, char *sPlayer)
 		setPlayerName(i, ""); //Clear custom names (use default ones instead)
 	}
 
-
+//	FIXME: FIX ME!!! **RANDOM** ARGH!
 	pl = rand()%game.maxPlayers;						//pick a player
 
 	player2dpid[pl] = NetPlay.dpidPlayer;				// add ourselves to the array.
@@ -429,14 +433,12 @@ BOOL LobbyLaunched(void)
 #endif
 
 // ////////////////////////////////////////////////////////////////////////////
-// say goodbye to everyone else
+// Broadcast that we are leaving the game 'nicely', (we wanted to) and not
+// because we have some kind of error. (dropped or disconnected)
 BOOL sendLeavingMsg(void)
 {
-	/*
-	 * Send a leaving message, This resolves a problem with tcpip which
-	 * occasionally doesn't automatically notice a player leaving
-	 */
-	NETbeginEncode(NET_LEAVING, NET_ALL_PLAYERS);
+	debug(LOG_NET, "We are leaving 'nicely'");
+	NETbeginEncode(NET_PLAYER_LEAVING, NET_ALL_PLAYERS);
 	{
 		uint32_t player_id = player2dpid[selectedPlayer];
 		BOOL host = NetPlay.bHost;
@@ -546,13 +548,29 @@ BOOL multiTemplateSetup(void)
 
 	case SKIRMISH:
 		// create the pc player list in deathmatch set.
+
+		// Yes, this code really *is* as hacky as it looks.
+		// I'm going to try to narrate what's going on.
+
+		// Player 5 currently contains the human player template set.
+		// (i.e. Nothing but a truck)
+		// Templates from players 5, 2, and 6 are added to player 4.
+		// This puts the AI template set in player 4.
 		addTemplateSet(CAMPAIGNTEMPLATES,DEATHMATCHTEMPLATES);
 		addTemplateSet(6,DEATHMATCHTEMPLATES);
 		addTemplateSet(2,DEATHMATCHTEMPLATES);
 
+		// To reiterate:
+		// Player 5 contains the human player template set.
+		// Player 4 contains the AI player template set.
+
 		//choose which way to do this.
 		if(isHumanPlayer(CAMPAIGNTEMPLATES))
 		{
+			// Player 5 is a human. No workarounds are needed.
+			// We just copy player 4's templates onto all the AI's,
+			// then copy player 5's template onto all the humans.
+
 			//pc first
 			for(player=0;player<game.maxPlayers;player++)
 			{
@@ -572,6 +590,13 @@ BOOL multiTemplateSetup(void)
 		}
 		else
 		{
+			// Player 5 is an AI player. This is a really bad workaround.
+			// We find an AI player: pcPlayer.
+			// We copy player 4's templates onto pcPlayer.
+			// Then, we copy player 5's templates onto all the
+			// human players, and pcPlayer's templates onto all the
+			// AI players.
+
 			// ensure a copy of pc templates to a pc player.
 			if(isHumanPlayer(DEATHMATCHTEMPLATES))
 			{
@@ -758,7 +783,6 @@ static BOOL campInit(void)
 {
 	UDWORD			player;
 	UBYTE		newPlayerArray[MAX_PLAYERS];
-	UDWORD		i,j,lastAI;
 	SDWORD		newPlayerTeam[MAX_PLAYERS] = {-1,-1,-1,-1,-1,-1,-1,-1};
 
 // if this is from a savegame, stop here!
@@ -777,35 +801,45 @@ static BOOL campInit(void)
 	//Convert skirmish GUI player ids to in-game ids
 	if(game.type == SKIRMISH)
 	{
-		lastAI = 0;		//last used AI slot
+		unsigned int i;
+		unsigned int lastAI = 0; // last used AI slot
+
 		memset(newPlayerArray,1,MAX_PLAYERS * sizeof(newPlayerArray[0]));		//'1' for humans
-		for(i=0;i<MAX_PLAYERS;i++)
+		for(i = 0; i < MAX_PLAYERS; ++i)
 		{
 			if(game.skDiff[i] < UBYTE_MAX )		//slot with enabled or disabled AI
 			{
+				unsigned int player;
+
 				//find first unused slot
-				for(j=lastAI;j<MAX_PLAYERS && isHumanPlayer(j);j++);	//skip humans
+				for (player = lastAI;
+				     player < MAX_PLAYERS && isHumanPlayer(player); // skip humans
+				     ++player);
 
-				ASSERT(j<MAX_PLAYERS,"campInit: couldn't find free slot while assigning AI %d , lastAI=%d", i, lastAI);
+				ASSERT(player < MAX_PLAYERS, "couldn't find free slot while assigning AI %u, lastAI=%u", i, lastAI);
 
-				newPlayerArray[j] = game.skDiff[i];		//copy over
-				newPlayerTeam[j] = playerTeamGUI[i];
+				newPlayerArray[player] = game.skDiff[i];		//copy over
+				newPlayerTeam[player] = playerTeamGUI[i];
 
 				//remove player if it was disabled in menus
 				if(game.skDiff[i] == 0)
-					clearPlayer(j,true,false);
+					clearPlayer(player,true,false);
 
-				lastAI = j;
+				lastAI = player;
 				lastAI++;
 			}
 			else if(game.skDiff[i] == UBYTE_MAX)	//human player
 			{
+				unsigned int player;
+
 				//find player net id
-				for(j=0;(j < MAX_PLAYERS) && (player2dpid[j] != NetPlay.players[i].dpid);j++);
+				for (player = 0;
+				     player < MAX_PLAYERS && player2dpid[player] != NetPlay.players[i].dpid;
+				     ++player);
 
-				ASSERT(j<MAX_PLAYERS,"campInit: couldn't find player id for GUI id %d", i);
+				ASSERT(player < MAX_PLAYERS, "couldn't find player id for GUI id %u", i);
 
-				newPlayerTeam[j] = playerTeamGUI[i];
+				newPlayerTeam[player] = playerTeamGUI[i];
 			}
 
 		}
@@ -889,6 +923,8 @@ BOOL multiGameShutdown(void)
 {
 	PLAYERSTATS	st;
 
+	debug(LOG_NET,"%s is shutting down.",getPlayerName(selectedPlayer));
+
 	sendLeavingMsg();							// say goodbye
 	updateMultiStatsGames();					// update games played.
 
